/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_SW64_SYSCALL_H
#define _ASM_SW64_SYSCALL_H

#include <uapi/linux/audit.h>

extern void *sys_call_table[];
static inline int syscall_get_nr(struct task_struct *task,
				 struct pt_regs *regs)
{
	return regs->r0;
}


static inline long syscall_get_return_value(struct task_struct *task,
					    struct pt_regs *regs)
{
	return regs->r0;
}

static inline void syscall_set_return_value(struct task_struct *task,
					    struct pt_regs *regs,
					    int error, long val)
{
	if (error) {
		regs->r0  = -error;
		regs->r19 = -1;
	} else {
		regs->r0 = val;
		regs->r19 = 0;
	}
}


static inline void syscall_rollback(struct task_struct *task,
				    struct pt_regs *regs)
{
	/* Do nothing */
}

static inline void syscall_get_arguments(struct task_struct *task,
					 struct pt_regs *regs, unsigned int i,
					 unsigned int n, unsigned long *args)
{
		switch (i) {
		case 0:
			if (!n--)
				break;
			*args++ = regs->r16;
		case 1:
			if (!n--)
				break;
			*args++ = regs->r17;
		case 2:
			if (!n--)
				break;
			*args++ = regs->r18;
		case 3:
			if (!n--)
				break;
			*args++ = regs->r19;
		case 4:
			if (!n--)
				break;
			*args++ = regs->r20;
		case 5:
			if (!n--)
				break;
			*args++ = regs->r21;
		case 6:
			if (!n--)
				break;
		default:
			BUG();
			break;
		}
}

static inline void syscall_set_arguments(struct task_struct *task,
					 struct pt_regs *regs, unsigned int i,
					 unsigned int n,
					 const unsigned long *args)
{
		switch (i) {
		case 0:
			if (!n--)
				break;
			regs->r16 = *args++;
		case 1:
			if (!n--)
				break;
			regs->r17 = *args++;
		case 2:
			if (!n--)
				break;
			regs->r18 = *args++;
		case 3:
			if (!n--)
				break;
			regs->r19 = *args++;
		case 4:
			if (!n--)
				break;
			regs->r20 = *args++;
		case 5:
			if (!n--)
				break;
			regs->r21 = *args++;
		case 6:
			if (!n--)
				break;
		default:
			BUG();
			break;
		}
}

static inline int syscall_get_arch(void)
{
	return AUDIT_ARCH_SW64;
}

#endif /* _ASM_SW64_SYSCALL_H */
